@@ -34,15 +34,13 @@
 typedef float                          vorbis_sample_t;
 typedef ogg_packet                     ogg_packet_wrapper;
 
-#define GST_VORBIS_DEC_ELEMENT_DETAILS \
-GST_ELEMENT_DETAILS ("Vorbis audio decoder",       \
-    "Codec/Decoder/Audio",                         \
-    "decode raw vorbis streams to float audio",    \
-    "Benjamin Otte <in7y118@public.uni-hamburg.de>")
+#define GST_VORBIS_DEC_DESCRIPTION "decode raw vorbis streams to float audio"
 
 #define GST_VORBIS_DEC_SRC_CAPS \
-    GST_STATIC_CAPS ("audio/x-raw-float, " "rate = (int) [ 1, MAX ], " \
-        "channels = (int) [ 1, 256 ], " "endianness = (int) BYTE_ORDER, " \
+    GST_STATIC_CAPS ("audio/x-raw-float, " \
+        "rate = (int) [ 1, MAX ], " \
+        "channels = (int) [ 1, 256 ], " \
+        "endianness = (int) BYTE_ORDER, " \
         "width = (int) 32")
 
 #define GST_VORBIS_DEC_DEFAULT_SAMPLE_WIDTH           (32)
@@ -87,12 +85,7 @@ struct _ogg_packet_wrapper {
   ogg_buffer          buf;
 };
 
-#define GST_VORBIS_DEC_ELEMENT_DETAILS \
-GST_ELEMENT_DETAILS ("Vorbis audio decoder",           \
-    "Codec/Decoder/Audio",                             \
-    "decode raw vorbis streams to integer audio",      \
-    "Benjamin Otte <in7y118@public.uni-hamburg.de>\n"  \
-    "Chris Lord <chris@openedhand.com>")
+#define GST_VORBIS_DEC_DESCRIPTION "decode raw vorbis streams to integer audio"
 
 #define GST_VORBIS_DEC_SRC_CAPS \
     GST_STATIC_CAPS ("audio/x-raw-int, "   \
@@ -100,7 +93,8 @@ GST_ELEMENT_DETAILS ("Vorbis audio decoder",           \
         "channels = (int) [ 1, 6 ], "      \
         "endianness = (int) BYTE_ORDER, "  \
         "width = (int) { 16, 32 }, "       \
-        "depth = (int) 16, " "signed = (boolean) true")
+        "depth = (int) 16, "               \
+        "signed = (boolean) true")
 
 #define GST_VORBIS_DEC_DEFAULT_SAMPLE_WIDTH           (16)
 
@@ -156,8 +150,9 @@ gst_ogg_packet_from_wrapper (ogg_packet_wrapper * packet)
 
 #endif
 
-void  copy_samples        (vorbis_sample_t *out, vorbis_sample_t **in,
+typedef void (*CopySampleFunc)(vorbis_sample_t *out, vorbis_sample_t **in,
                            guint samples, gint channels, gint width);
 
+CopySampleFunc get_copy_sample_func (gint channels, gint width);
 
 #endif /* __GST_VORBIS_DEC_LIB_H__ */
