@@ -32,13 +32,14 @@
  * their base_init function to easily configure the set of caps/formats that
  * the element is able to handle.
  *
- * Derived classes should override the GstAudioFilter::setup() and
- * GstBaseTransform::transform_ip() and/or GstBaseTransform::transform()
+ * Derived classes should override the #GstAudioFilterClass.setup() and
+ * #GstBaseTransformClass.transform_ip() and/or
+ * #GstBaseTransformClass.transform()
  * virtual functions in their class_init function.
  *
- * Since: 0.10.12
- *
  * Last reviewed on 2007-02-03 (0.10.11.1)
+ *
+ * Since: 0.10.12
  */
 
 #ifdef HAVE_CONFIG_H
@@ -52,8 +53,6 @@
 GST_DEBUG_CATEGORY_STATIC (audiofilter_dbg);
 #define GST_CAT_DEFAULT audiofilter_dbg
 
-static void gst_audio_filter_class_init (gpointer g_class, gpointer class_data);
-static void gst_audio_filter_init (GTypeInstance * instance, gpointer g_class);
 static GstStateChangeReturn gst_audio_filter_change_state (GstElement * element,
     GstStateChange transition);
 static gboolean gst_audio_filter_set_caps (GstBaseTransform * btrans,
@@ -61,44 +60,18 @@ static gboolean gst_audio_filter_set_caps (GstBaseTransform * btrans,
 static gboolean gst_audio_filter_get_unit_size (GstBaseTransform * btrans,
     GstCaps * caps, guint * size);
 
-static GstElementClass *parent_class = NULL;
+#define do_init G_STMT_START { \
+    GST_DEBUG_CATEGORY_INIT (audiofilter_dbg, "audiofilter", 0, "audiofilter"); \
+} G_STMT_END
 
-GType
-gst_audio_filter_get_type (void)
-{
-  static GType audio_filter_type = 0;
-
-  if (!audio_filter_type) {
-    const GTypeInfo audio_filter_info = {
-      sizeof (GstAudioFilterClass),
-      NULL,
-      NULL,
-      gst_audio_filter_class_init,
-      NULL,
-      NULL,
-      sizeof (GstAudioFilter),
-      0,
-      gst_audio_filter_init,
-    };
-
-    GST_DEBUG_CATEGORY_INIT (audiofilter_dbg, "audiofilter", 0, "audiofilter");
-
-    audio_filter_type = g_type_register_static (GST_TYPE_BASE_TRANSFORM,
-        "GstAudioFilter", &audio_filter_info, G_TYPE_FLAG_ABSTRACT);
-  }
-  return audio_filter_type;
-}
+G_DEFINE_ABSTRACT_TYPE_WITH_CODE (GstAudioFilter, gst_audio_filter,
+    GST_TYPE_BASE_TRANSFORM, do_init);
 
 static void
-gst_audio_filter_class_init (gpointer klass, gpointer class_data)
+gst_audio_filter_class_init (GstAudioFilterClass * klass)
 {
-  GstBaseTransformClass *basetrans_class;
-  GstElementClass *gstelement_class;
-
-  parent_class = g_type_class_peek_parent (klass);
-
-  gstelement_class = (GstElementClass *) klass;
-  basetrans_class = (GstBaseTransformClass *) klass;
+  GstBaseTransformClass *basetrans_class = (GstBaseTransformClass *) klass;
+  GstElementClass *gstelement_class = (GstElementClass *) klass;
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_audio_filter_change_state);
@@ -114,7 +87,7 @@ gst_audio_filter_class_init (gpointer klass, gpointer class_data)
 }
 
 static void
-gst_audio_filter_init (GTypeInstance * instance, gpointer g_class)
+gst_audio_filter_init (GstAudioFilter * self)
 {
   /* nothing to do here */
 }
@@ -126,9 +99,7 @@ static GstStateChangeReturn
 gst_audio_filter_change_state (GstElement * element, GstStateChange transition)
 {
   GstStateChangeReturn ret;
-  GstAudioFilter *filter;
-
-  filter = GST_AUDIO_FILTER (element);
+  GstAudioFilter *filter = GST_AUDIO_FILTER (element);
 
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
@@ -140,7 +111,9 @@ gst_audio_filter_change_state (GstElement * element, GstStateChange transition)
       break;
   }
 
-  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+  ret =
+      GST_ELEMENT_CLASS (gst_audio_filter_parent_class)->change_state (element,
+      transition);
   if (ret == GST_STATE_CHANGE_FAILURE)
     return ret;
 
@@ -161,13 +134,9 @@ gst_audio_filter_set_caps (GstBaseTransform * btrans, GstCaps * incaps,
     GstCaps * outcaps)
 {
   GstAudioFilterClass *klass;
-  GstAudioFilter *filter;
+  GstAudioFilter *filter = GST_AUDIO_FILTER (btrans);
   gboolean ret = TRUE;
 
-  g_assert (gst_caps_is_equal (incaps, outcaps));
-
-  filter = GST_AUDIO_FILTER (btrans);
-
   GST_LOG_OBJECT (filter, "caps: %" GST_PTR_FORMAT, incaps);
 
   if (!gst_ring_buffer_parse_caps (&filter->format, incaps)) {
@@ -175,7 +144,7 @@ gst_audio_filter_set_caps (GstBaseTransform * btrans, GstCaps * incaps,
     return FALSE;
   }
 
-  klass = GST_AUDIO_FILTER_CLASS (G_OBJECT_GET_CLASS (filter));
+  klass = GST_AUDIO_FILTER_CLASS_CAST (G_OBJECT_GET_CLASS (filter));
 
   if (klass->setup)
     ret = klass->setup (filter, &filter->format);
@@ -219,16 +188,18 @@ gst_audio_filter_class_add_pad_templates (GstAudioFilterClass * klass,
     const GstCaps * allowed_caps)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+  GstPadTemplate *pad_template;
 
   g_return_if_fail (GST_IS_AUDIO_FILTER_CLASS (klass));
-  g_return_if_fail (allowed_caps != NULL);
   g_return_if_fail (GST_IS_CAPS (allowed_caps));
 
-  gst_element_class_add_pad_template (element_class,
-      gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
-          gst_caps_copy (allowed_caps)));
+  pad_template = gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
+      gst_caps_copy (allowed_caps));
+  gst_element_class_add_pad_template (element_class, pad_template);
+  gst_object_unref (pad_template);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_pad_template_new ("sink", GST_PAD_SINK, GST_PAD_ALWAYS,
-          gst_caps_copy (allowed_caps)));
+  pad_template = gst_pad_template_new ("sink", GST_PAD_SINK, GST_PAD_ALWAYS,
+      gst_caps_copy (allowed_caps));
+  gst_element_class_add_pad_template (element_class, pad_template);
+  gst_object_unref (pad_template);
 }
