@@ -62,17 +62,24 @@ typedef struct
 static const FormatInfo formats[] = {
   /* container/tag formats with static descriptions */
   {"application/ogg", "Ogg", FLAG_CONTAINER},
+  {"application/mxf", "MXF", FLAG_CONTAINER},
   {"application/vnd.rn-realmedia", "Realmedia", FLAG_CONTAINER},
   {"application/x-id3", N_("ID3 tag"), FLAG_CONTAINER},
   {"application/x-ape", N_("APE tag"), FLAG_CONTAINER},
+  {"application/x-apetag", N_("APE tag"), FLAG_CONTAINER},
   {"application/x-icy", N_("ICY internet radio"), FLAG_CONTAINER},
+  {"application/x-3gp", "3GP", FLAG_CONTAINER},
+  {"application/x-yuv4mpeg", "Y4M", FLAG_CONTAINER},
+  {"multipart/x-mixed-replace", "Multipart", FLAG_CONTAINER},
   {"video/x-fli", "FLI/FLC/FLX Animation", FLAG_CONTAINER},
   {"video/x-flv", "Flash", FLAG_CONTAINER},
   {"video/x-matroska", "Matroska", FLAG_CONTAINER},
+  {"video/webm", "WebM", FLAG_CONTAINER},
   {"video/x-ms-asf", "Advanced Streaming Format (ASF)", FLAG_CONTAINER},
   {"video/x-msvideo", "AVI", FLAG_CONTAINER},
   {"video/x-quicktime", "Quicktime", FLAG_CONTAINER},
   {"video/quicktime", "Quicktime", FLAG_CONTAINER},
+  {"video/mj2", "Motion JPEG 2000", FLAG_CONTAINER},
 
   /* audio formats with static descriptions */
   {"audio/x-ac3", "AC-3 (ATSC A/52)", 0},
@@ -119,6 +126,7 @@ static const FormatInfo formats[] = {
   {"audio/x-shorten", "Shorten Lossless", 0},
   {"audio/x-sid", "Sid", 0},
   {"audio/x-sipro", "Sipro/ACELP.NET Voice", 0},
+  {"audio/x-siren", "Siren", 0},
   {"audio/x-spc", "SNES-SPC700 Sound File Data", 0},
   {"audio/x-speex", "Speex", 0},
   {"audio/x-svx", "Amiga IFF / SVX8 / SV16", 0},
@@ -141,6 +149,7 @@ static const FormatInfo formats[] = {
   {"video/x-3ivx", "3ivx", 0},
   {"video/x-4xm", "4X Techologies Video", 0},
   {"video/x-apple-video", "Apple video", 0},
+  {"video/x-aasc", "Autodesk Animator", 0},
   {"video/x-camtasia", "TechSmith Camtasia", 0},
   {"video/x-cdxa", "RIFF/CDXA (VCD)", 0},
   {"video/x-cinepak", "Cinepak Video", 0},
@@ -156,6 +165,7 @@ static const FormatInfo formats[] = {
   {"video/x-intel-h263", "Intel H.263", 0},
   {"video/x-jpeg", "Motion JPEG", 0},
   /* { "video/x-jpeg-b", "", 0 }, does this actually exist? */
+  {"video/x-loco", "LOCO Lossless", 0},
   {"video/x-mjpeg", "Motion-JPEG", 0},
   {"video/x-mjpeg-b", "Motion-JPEG format B", 0},
   {"video/mpegts", "MPEG-2 Transport Stream", FLAG_CONTAINER},
@@ -180,8 +190,10 @@ static const FormatInfo formats[] = {
   {"video/x-vp6", "On2 VP6", 0},
   {"video/x-vp6-flash", "On2 VP6/Flash", 0},
   {"video/x-vp7", "On2 VP7", 0},
+  {"video/x-vp8", "VP8", 0},
   {"video/x-xvid", "XVID MPEG-4", 0},
   {"video/x-zlib", "Lossless zlib video", 0},
+  {"video/x-zmbv", "Zip Motion Block video", 0},
 
   /* image formats with static descriptions */
   {"image/bmp", "BMP", 0},
@@ -197,6 +209,9 @@ static const FormatInfo formats[] = {
   {"image/tiff", "TIFF", 0},
   {"image/x-cmu-raster", "CMU Raster Format", 0},
   {"image/x-icon", "ICO", 0},
+  {"image/x-j2c", "JPEG 2000", 0},
+  {"image/x-jpc", "JPEG 2000", 0},
+  {"image/jp2", "JPEG 2000", 0},
   {"image/x-xcf", "XFC", 0},
   {"image/x-pixmap", "XPM", 0},
   {"image/x-xpixmap", "XPM", 0},
@@ -207,9 +222,14 @@ static const FormatInfo formats[] = {
   /* subtitle formats with static descriptions */
   {"application/x-subtitle-sami", N_("Sami subtitle format"), 0},
   {"application/x-subtitle-tmplayer", N_("TMPlayer subtitle format"), 0},
-  {"application/x-kate", N_("Kate subtitle format"), 0},
+  {"application/x-kate", "Kate", 0},
+  {"subtitle/x-kate", N_("Kate subtitle format"), 0},
   /* add variant field to typefinder? { "application/x-subtitle", N_("subtitle"), 0}, */
 
+  /* non-audio/video/container formats */
+  {"hdv/aux-v", "HDV AUX-V", 0},
+  {"hdv/aux-a", "HDV AUX-A", 0},
+
   /* formats with dynamic descriptions */
   {"audio/mpeg", NULL, 0},
   {"audio/x-adpcm", NULL, 0},
