@@ -219,6 +219,25 @@ gboolean                gst_tag_list_add_id3_image (GstTagList   * tag_list,
                                                     guint          image_data_len,
                                                     guint          id3_picture_type);
 
+/* functions to  convert GstBuffers with xmp packets contents to GstTagLists and back */
+GstTagList *            gst_tag_list_from_xmp_buffer  (const GstBuffer *  buffer);
+GstBuffer *             gst_tag_list_to_xmp_buffer    (const GstTagList * list,
+                                                       gboolean           read_only);
+
+/* functions related to exif */
+GstBuffer *             gst_tag_list_to_exif_buffer (const GstTagList * taglist,
+                                                     gint byte_order,
+                                                     guint32 base_offset);
+
+GstBuffer *             gst_tag_list_to_exif_buffer_with_tiff_header (const GstTagList * taglist);
+
+GstTagList *            gst_tag_list_from_exif_buffer (const GstBuffer * buffer,
+                                                       gint byte_order,
+                                                       guint32 base_offset);
+
+GstTagList *            gst_tag_list_from_exif_buffer_with_tiff_header (
+                                                      const GstBuffer * buffer);
+
 /* other tag-related functions */
 
 gboolean                gst_tag_parse_extended_comment (const gchar  * ext_comment,
@@ -238,6 +257,30 @@ GstBuffer             * gst_tag_image_data_to_image_buffer (const guint8   * ima
 /* FIXME 0.11: replace with a more general gst_tag_library_init() */
 void                    gst_tag_register_musicbrainz_tags (void);
 
+
+/* language tag related functions */
+
+gchar **       gst_tag_get_language_codes (void);
+
+const gchar *  gst_tag_get_language_name (const gchar * language_code);
+
+const gchar *  gst_tag_get_language_code_iso_639_1 (const gchar * lang_code);
+
+const gchar *  gst_tag_get_language_code_iso_639_2B (const gchar * lang_code);
+
+const gchar *  gst_tag_get_language_code_iso_639_2T (const gchar * lang_code);
+
+/**
+ * gst_tag_get_language_code:
+ * @lang_code: ISO-639 language code (e.g. "deu" or "ger" or "de")
+ *
+ * Convenience macro wrapping gst_tag_get_language_code_iso_639_1().
+ *
+ * Since: 0.10.26
+ */
+#define gst_tag_get_language_code(lang_code) \
+    gst_tag_get_language_code_iso_639_1(lang_code)
+
 G_END_DECLS
 
 #endif /* __GST_TAG_TAG_H__ */
